-------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;

use work.emp_data_types.all;
use work.mprocessor_constants.all;


entity EcalDataAggregator is
port (
       --- Input Ports ---
        clk_p            : in  std_logic;
        data_in          : in tDaqEcalLinkDataArray;
        --- Output Ports ---
        data_out_start   : out std_logic; -- '1' for one clock when the data_out is valid and updated. (should be 'start'?)
        data_out_last    : out std_logic;
        data_out         : out std_logic_vector(28 * 16 -1 downto 0)
);
end EcalDataAggregator;

architecture rtl of EcalDataAggregator is

  signal ecal_data                   : std_logic_vector(28 * 16 -1 downto 0);
  alias padding                      is ecal_data(ecal_data'high downto ecal_data'high-2);
  alias bcid                         is ecal_data(ecal_data'high-4 downto ecal_data'high-4-11);
  
  signal ecal_data_p                 : std_logic_vector(28 * 16 -1 downto 0);
  alias padding_p                    is ecal_data(ecal_data'high downto ecal_data'high-2);
  
  signal data_out_s                  : std_logic_vector(28 * 16 -1 downto 0) := (others => '0');
  signal data_out_s_start            : std_logic := '0'; 
  signal data_out_s_last             : std_logic := '0';

  signal sync                        : std_logic := '0';
  
  type tBcIdPipe is array (integer range 0 to 8) of std_logic_vector(11 downto 0);
  signal bcid_p : tBcIdPipe;
  signal last_valid_bcid : std_logic_vector(11 downto 0);

begin
  
  EclDataFormatter : for i in 0 to 3 generate
    type tEclLinkToGroupIndexArray is array (integer range 0 to 3) of integer;
    constant ecal_link_to_group_indices   :  tEclLinkToGroupIndexArray := (1,0,2,3);
  begin
 
    -- rearanging data
    ch : for j in 0 to 6 generate
      signal chid  : integer range 0 to 27;
    begin
      
      chid <= i+j*4;
      ecal_data((chid+1)*16-1 downto chid *16) <= data_in(ecal_link_to_group_indices(i))(j);
    
    end generate ch;
  
  end generate EclDataFormatter;
  
  
  data_check: process(clk_p)
    variable timer : integer range 0 to 7 := 0;
    begin
  
        if rising_edge(clk_p) then
          
          -- ecal_data pipeline
          ecal_data_p <=  ecal_data;
          bcid_p(0) <= bcid;
          for i in 0 to 7 loop
            bcid_p(i+1) <= bcid_p(i);
          end loop;
                           
					-- data_out pipeline
					data_out <= data_out_s;
          data_out_start <= data_out_s_start;
          data_out_last <= data_out_s_last;

          -- capturing frames
          data_out_s_start <= '0';
          data_out_s_last <= '0';
          if sync = '0' then
             if padding_p = "111" and (unsigned(bcid_p(0)) = unsigned(bcid_p(8)) + 1 or (unsigned(bcid_p(0)) = 0 and unsigned(bcid_p(8)) = 3563) ) then
               timer := 0;
               data_out_s <= ecal_data_p;
               data_out_s_start <= '1';
               last_valid_bcid <= bcid_p(0);
               sync <= '1';
             elsif timer = 7 then
               timer := 0;
             else
               timer := timer + 1;
             end if;
          else
            if timer = 7 then
              timer := 0;
              if padding_p = "111" and (unsigned(bcid_p(0)) = unsigned(bcid_p(8)) + 1 or (unsigned(bcid_p(0)) = 0 and unsigned(bcid_p(8)) = 3563) ) then
                data_out_s <= ecal_data_p;
                data_out_s_start <= '1'; 
                last_valid_bcid <= bcid_p(0);
              else
                sync <= '0';
              end if;
            else 
               timer := timer + 1;
               if timer = 7 then
                 data_out_s_last <= '1';
               end if;
            end if;
          end if;

        end if;
    end process;

end rtl;

