library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;

use work.emp_data_types.all;

package tracking_constants is
    
    type tTrackStubs is array (0 to 5) of lword;
    
    type tStub is record
        z : std_logic_vector(3 downto 0);
        bend : std_logic_vector(3 downto 0);
        address : std_logic_vector(7 downto 0);
        cbc : std_logic_vector(2 downto 0);
        cic : std_logic_vector(0 downto 0);
    end record;
    constant cNullStub : tStub := ((others => '0'), (others => '0'), (others => '0'), (others => '0'), (others => '0'));
    type tStubArray is array(0 to 5) of tStub;
    
    function convertSLVtoStub(word_in : in std_logic_vector(63 downto 0)) return tStub;
    
    type tCICMappingArray is array (0 to 7) of integer;
    constant cCICMapping : tCICMappingArray := (0, 1, 2, 3, 7, 6, 5, 4);


    
    type tDecodedStub is record
        localx : integer;
        localy : integer;
    end record;
    constant cNullDecodedStub : tDecodedStub := (0, 0);
    type tDecodedStubArray is array(0 to 5) of tDecodedStub;
    
    function convertStubtoDecodedStub(stub_in: tStub) return tDecodedStub;
    
end package tracking_constants;


package body tracking_constants is

    function convertSLVtoStub(word_in : in std_logic_vector(63 downto 0)) return tStub is
        variable stub : tStub;
    begin
        stub.z := word_in(3 downto 0);
        stub.bend := word_in(7 downto 4);
        stub.address := word_in(15 downto 8);
        stub.cbc := word_in(18 downto 16);
        stub.cic := word_in(50 downto 50);
        return stub;
    end convertSLVtoStub;
    
    function convertStubtoDecodedStub(stub_in: tStub) return tDecodedStub is
        variable decoded_stub : tDecodedStub;
    begin
        if stub_in.cic = "1" then
            decoded_stub.localy := 1;
            decoded_stub.localx := 2032 - to_integer(unsigned(stub_in.address)) - 254*cCICMapping(to_integer(unsigned(stub_in.cbc))) + 2;
        else
            decoded_stub.localy := 0;
            decoded_stub.localx := to_integer(unsigned(stub_in.address)) + 254* cCICMapping(to_integer(unsigned(stub_in.cbc)));
        end if;
        return decoded_stub;
    end convertStubtoDecodedStub;

end tracking_constants;
