-- emp_project_decl
--
-- Defines constants for the whole device

library ieee;
use ieee.std_logic_1164.all;

library lpgbt_lib;
use lpgbt_lib.lpgbtfpga_package.all;

use work.emp_framework_decl.all;
use work.emp_device_decl.all;
use work.emp_device_types.all;
use work.emp_lpgbt_decl.all;
use work.emp_data_framer_decl.all;
use work.emp_slink_types.all;
use work.emp_eth10g_types.all;

-------------------------------------------------------------------------------
package emp_project_decl is

  constant PAYLOAD_REV         : std_logic_vector(31 downto 0) := X"d451d001";

  -- Latency buffer size
  constant LB_ADDR_WIDTH      : integer               := 10;

  -- Clock setup
  constant CLOCK_COMMON_RATIO : integer               := 32;
  constant CLOCK_RATIO        : integer               := 8;
  constant CLOCK_AUX_DIV      : clock_divisor_array_t := (16, 8, 4); -- Dividers of CLOCK_COMMON_RATIO * 40 MHz

  -- Only used by nullalgo
  constant PAYLOAD_LATENCY    : integer             := 5;

  -- mgt -> buf -> fmt -> (algo) -> (fmt) -> buf -> mgt -> clk -> altclk
  constant REGION_CONF : region_conf_array_t := (
  --  0  => (no_mgt, buf, no_fmt, buf, no_mgt),   --Bank 225 -- Right Column
    1  => (no_mgt, buf, no_fmt, buf, no_mgt),   --Bank 226
  --  2  => (lpgbt, buf, no_fmt, buf, lpgbt),   --Bank 227
    3  => (lpgbt, buf, no_fmt, buf, lpgbt),   --Bank 228
    4  => (lpgbt, buf, no_fmt, buf, lpgbt),   --Bank 229
    5  => (lpgbt, buf, no_fmt, buf, lpgbt),   --Bank 230
    6  => (lpgbt, buf, no_fmt, buf, lpgbt),   --Bank 231
    7  => (no_mgt, buf, no_fmt, buf, no_mgt),   --Bank 232
    8  => (no_mgt, buf, no_fmt, buf, no_mgt),   --Bank 233
    9  => (no_mgt, buf, no_fmt, buf, no_mgt),   --Bank 234
--    -- Cross-chip
-- KH used for gbe
    --10     => (gty25, buf, no_fmt, buf, gty25),
    --11     => (gty25, buf, no_fmt, buf, gty25),
    --12     => (gty25, buf, no_fmt, buf, gty25),
-- KH used for gbe
    13 => (no_mgt, buf, no_fmt, buf, no_mgt),   --Bank 131
    14 => (no_mgt, buf, no_fmt, buf, no_mgt),   --Bank 130
    15 => (no_mgt, buf, no_fmt, buf, no_mgt),   --Bank 129
--    16 => (gty25, buf, no_fmt, buf, gty25),   --Bank 128
--    17 => (gty25, buf, no_fmt, buf, gty25),   --Bank 127
      others => kDummyRegion
  );

  -- for data framer (ic_simple, no_ec, n_ec_spare, ec_broadcast)
  constant REGION_DATA_FRAMER_CONF : region_data_framer_conf_array_t := (
--    2 => ( 0=>(false, true, 0, false, lpgbtv0), 1=>(false, true, 0, false, lpgbtv0), 2=>(false, true, 0, false, lpgbtv1), 3=>(false, true, 0, false, lpgbtv0)),
--    3 => ( 0=>(false, true, 0, false, lpgbtv0), 1=>(false, true, 0, false, lpgbtv0), 2=>(false, true, 0, false, lpgbtv0), 3=>(false, true, 0, false, lpgbtv0)),
    3 => ( 
      0=>(false, true, 0, false, lpgbtv0),
      1=>(false, true, 0, false, lpgbtv0),
      2=>(false, true, 0, false, lpgbtv0),
      3=>(false, true, 0, false, lpgbtv0)
    ),
    4 => ( 
      0=>(false, true, 0, false, lpgbtv0),
      1=>(false, true, 0, false, lpgbtv0),
      2=>(false, true, 0, false, lpgbtv0),
      3=>(false, true, 0, false, lpgbtv0)
    ),
    5 => (
      0=>(false, true, 0, false, lpgbtv1),
      1=>(false, true, 0, false, lpgbtv0),
      2=>(false, true, 0, false, lpgbtv0),
      3=>(false, true, 0, false, lpgbtv1)
    ),
    6 => ( 
      0=>(false, true, 0, false, lpgbtv0),
      1=>(false, true, 0, false, lpgbtv0),
      2=>(false, true, 0, false, lpgbtv0),
      3=>(false, true, 0, false, lpgbtv1)
    ),
    others => kDummyRegionDataFramer
  );

  -- for lpgbt
  constant REGION_LPGBT_CONF : region_lpgbt_conf_array_t := (
--    2  => (FEC5, DATARATE_5G12, PCS),
    3  => (FEC5, DATARATE_5G12, PCS),
    4  => (FEC5, DATARATE_5G12, PCS),
    5  => (FEC5, DATARATE_5G12, PCS),
    6  => (FEC5, DATARATE_5G12, PCS),
    others => kDummyRegionLpgbt
  );

  -- Specify the slink quad using the corresponding region conf ID
  -- Specify slink channels to enable using the channel mask
  constant SLINK_CONF : slink_conf_array_t := (
    others => kNoSlink
  );

  constant ETH10G_CONF : eth10g_conf_array_t := (
    0 => (10, "1111", (x"aabbccddef51", x"aabbccddef52", x"aabbccddef53", x"aabbccddef54"))
  );



end emp_project_decl;
-------------------------------------------------------------------------------
